<?php

/**
 * @file
 * paybox recurring payment.
 *
 */

 /**
 * implements hook_recurring_info()
 * Define the recurring payment method/gateway function callbacks.
 */
function uc_paybox_recurring_info() {
  $items['paybox_direct'] = array(
    'name' => t('Paybox'),
    'payment method' => 'credit',
    'module' => 'uc_recurring',
    'fee handler' => 'paybox_direct',
    'renew callback' => 'uc_paybox_recurring_renew',
    'process callback' => 'uc_paybox_recurring_process',
    'saved profile' => TRUE,
    'menu' => array(
      'charge' => UC_RECURRING_MENU_DEFAULT,
      'edit'   => UC_RECURRING_MENU_DEFAULT,
      'cancel' => UC_RECURRING_MENU_DEFAULT,
    ), // Use the default user operation defined in uc_recurring.
  );
  return $items;
}

/**
 * Set up the recurring fee by creating a customer reference for future payments
 *
 * @param $order
 *   The order object.
 * @param $fee
 *   The fee object.
 * @return
 *   TRUE if recurring fee setup
 */
function uc_paybox_recurring_process($order, &$fee) {
  $paybox = db_fetch_object(db_query("SELECT * FROM {uc_paybox} WHERE uid = %d ORDER BY dateval DESC", $order->uid));
  $data = array(
    'TYPE' => '00053', // auth + debit on abonne
    'REFABONNE' => $paybox->refabonne,
    'PORTEUR' => $paybox->porteur,
    'DATEVAL' => $paybox->dateval,
    'CVV' => $paybox->cvv,
  );
  if (variable_get('uc_paybox_debug', FALSE) == TRUE) {
    watchdog('uc_paybox',
      'Process payment, type: !type, order: !orderid, abonne: !ref, amount: !amount',
      array(
        '!type' => $data['TYPE'],
        '!orderid' => $order->order_id,
        '!ref' => $paybox->refabonne,
        '!amount' => $order->order_total,
      ),
      WATCHDOG_DEBUG
    );
  }
  return _uc_paybox_charge($order, $order->order_total, $data);
}

/**
 * Process a renewal using the Paybox profile.
 *
 * @param $order
 *   The order object.
 * @param $fee
 *   The fee object.
 * @return
 *   TRUE if renewal succeeded
 */
function uc_paybox_recurring_renew($order, &$fee) {
  if (!empty($order->data['cc_txns']['references']['paybox_direct'])) {
    $data = array(
      'TYPE' => '00053', // auth + debit on abonne
      'REFABONNE' => $order->data['cc_txns']['references']['paybox_direct']['refabonne'],
      'PORTEUR' => $order->data['cc_txns']['references']['paybox_direct']['porteur'],
      'DATEVAL' => $order->data['cc_txns']['references']['paybox_direct']['dateval'],
      'CVV' => $order->data['cc_txns']['references']['paybox_direct']['cvv'],
      'REFERENCE' => $fee->order_id,
    );
    $result = _uc_paybox_charge($order->order_id, $order->order_total, $data);

    if ($result['success'] == TRUE) {
      uc_payment_enter($order->order_id, $order->payment_method, $order->order_total, $fee->uid, $result['data'], $result['comment']);
      if (variable_get('uc_paybox_debug', FALSE) == TRUE) {
        watchdog('uc_paybox',
          'Renew recurring, type: !type, order: !orderid, abonne: !ref, amount: !amount',
          array(
            '!type' => $data['TYPE'],
            '!orderid' => $order->order_id,
            '!ref' => $paybox->refabonne,
            '!amount' => $order->order_total,
          ),
          WATCHDOG_DEBUG
        );
      }
      return TRUE;
    }
    else {
      watchdog('uc_paybox', 'Renew failed: order: !orderid, fee id: !ref', array('!rderid' => $order->id, '!ref' => $fee->order_id), WATCHDOG_ERROR);
    }
  }
  else {
    watchdog('uc_paybox', 'Renew error, no data: order: !orderid, fee id: !ref', array('!rderid' => $order->id, '!ref' => $fee->order_id), WATCHDOG_ERROR);
  }
  return FALSE;
}

/**
 * Create a paybox abonne using an order's data.
 */
function _uc_paybox_profile_create($order, $amount) {
  if ($order->uid == 0) {
    return t('You must have an account to buy a recurring product.');
  }
  $data = array(
    'TYPE' => '00056', // Paybox create abonne
    'REFABONNE' => $order->uid,
    'NUMQUESTION' => $order->order_id . date('His'),
    'MONTANT' => number_format($amount, 2, variable_get('uc_currency_dec', '.'), '') * 100,
    'REFERENCE' => $order->order_id, // max 25 car.
    'PORTEUR' => $order->payment_details['cc_number'],
    'DATEVAL' => str_pad($order->payment_details['cc_exp_month'], 2, '0', STR_PAD_LEFT) . substr($order->payment_details['cc_exp_year'], -2),
    'CVV' => $order->payment_details['cc_cvv'],
  );
  $data += _uc_paybox_direct_build_datas($data);

  $response = _uc_paybox_direct_http_request($data);

  if (variable_get('uc_paybox_debug', FALSE) == TRUE) {
    watchdog('uc_paybox',
      'Profile create: !type, order: !orderid, amount: !amount, !code: !msg',
      array(
        '!type' => $data['TYPE'],
        '!orderid' => $order->order_id,
        '!amount' => $amount,
        '!code' => $response['CODEREPONSE'],
        '!msg' => _uc_paybox_error($response['CODEREPONSE']),
      ),
      WATCHDOG_DEBUG
    );
  }

  // check abonne
  if ($response['CODEREPONSE'] == '00016') {
    uc_order_comment_save($order->order_id, 0, t('Paybox: abonne already exist - @ref', array('@ref' => $response['REFABONNE'])));
  }
  elseif ($response['CODEREPONSE'] == '00000') {
    uc_order_comment_save($order->order_id, 0, t('Paybox: abonne created - @ref', array('@ref' => $response['REFABONNE'])));
  }
  else {
    uc_order_comment_save($order->order_id, 0, t('Paybox: Creating abonne failed.<br />@error - @text', array('@error' => $response['CODEREPONSE'], '@text' => _uc_paybox_error($response['CODEREPONSE']))), 'admin');
    // return error message
    return _uc_paybox_error($response['CODEREPONSE']);
  }
  // Save the new credit reference to the db.
  $order->data = uc_paybox_log_reference($order->order_id, $order->uid, $response, $data);

  return TRUE;
}

/**
 * Save paybox user datas on order and table.
 * Currently datas saved on order is not visible in uc_paybox_recurring_process()
 * that's why we need table storage.
 */
function uc_paybox_log_reference($order_id, $uid, $response, $datas) {
  // Save credit values to specific table
  db_query("INSERT INTO {uc_paybox} (order_id, uid, refabonne, porteur, dateval, cvv) VALUES (%d, %d, '%s', '%s', '%s', '%s')",
    $order_id,
    $uid,
    $response['REFABONNE'],
    $response['PORTEUR'],
    $datas['DATEVAL'],
    $datas['CVV']
  );
  // Load the existing order data array.
  $data = db_result(db_query("SELECT data FROM {uc_orders} WHERE order_id = %d", $order_id));
  $data = unserialize($data);

  $data['cc_txns']['references']['paybox_direct'] = array(
    'refabonne' => $response['REFABONNE'],
    'porteur' => $response['PORTEUR'],
    'dateval' => $datas['DATEVAL'],
    'cvv' => $datas['CVV'],
    'numtrans' => $response['NUMTRANS'],
    'created' => time(),
  );
  // Save the updated data array to the database.
  db_query("UPDATE {uc_orders} SET data = '%s' WHERE order_id = %d", serialize($data), $order_id);

  if (variable_get('uc_paybox_debug', FALSE) == TRUE) {
    watchdog('uc_paybox',
      'Log reference: !orderid, abonne: !ref',
      array(
        '!orderid' => $order_id,
        '!ref' => $response['REFABONNE'],
      ),
      WATCHDOG_DEBUG
    );
  }

  return $data;
}